@@ -2,7 +2,7 @@ package Imager::Fill;
 use strict;
 use vars qw($VERSION);
 
-$VERSION = sprintf "%d.%03d", q$Revision: 1.9 $=~/\d+/g;
+$VERSION = "1.010";
 
 # this needs to be kept in sync with the array of hatches in fills.c
 my @hatch_types =
@@ -170,8 +170,20 @@ sub combines {
 
 =head1 DESCRIPTION 
 
-Creates fill objects for use by some drawing functions, currently just
-the Imager box() method.
+Creates fill objects for use by most filled area drawing functions.
+
+All fills are created with the new method.
+
+=over
+
+=item new
+
+  my $fill = Imager::Fill->new(...);
+
+The parameters depend on the type of fill being created.  See below
+for details.
+
+=back
 
 The currently available fills are:
 
@@ -386,7 +398,7 @@ You can call Imager::Fill->hatches for a list of hatch names.
 
   my $fill = Imager::Fill->new(fountain=>$ftype, 
        xa=>$xa, ya=>$ya, xb=>$xb, yb=>$yb, 
-       segment=>$segments, repeat=>$repeat, combine=>$combine, 
+       segments=>$segments, repeat=>$repeat, combine=>$combine, 
        super_sample=>$super_sample, ssample_param=>$ssample_param);
 
 This fills the given region with a fountain fill.  This is exactly the
